package com.fjhx.victoriatourist.service.jd.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.victoriatourist.entity.jd.dto.JdRefundNotQualityCheckSelectDto;
import com.fjhx.victoriatourist.entity.jd.po.JdRefundNotQualityCheck;
import com.fjhx.victoriatourist.entity.jd.po.JdRefundQualityCheck;
import com.fjhx.victoriatourist.entity.jd.vo.JdRefundNotQualityCheckVo;
import com.fjhx.victoriatourist.mapper.jd.JdRefundNotQualityCheckMapper;
import com.fjhx.victoriatourist.service.jd.JdRefundNotQualityCheckService;
import com.fjhx.victoriatourist.service.jd.JdRefundQualityCheckService;
import com.fjhx.victoriatourist.utils.CodeEnum;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.service.stock.StockService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.fjhx.victoriatourist.service.jd.impl.JdRefundServiceImpl.WAREHOUSE_ID;
import static com.fjhx.victoriatourist.service.jd.impl.JdRefundServiceImpl.getUpdateStockSql;


/**
 * <p>
 * 京东售后退货待质检 服务实现类
 * </p>
 *
 * @author
 * @since 2024-09-28
 */
@Service
public class JdRefundNotQualityCheckServiceImpl extends ServiceImpl<JdRefundNotQualityCheckMapper, JdRefundNotQualityCheck> implements JdRefundNotQualityCheckService {

    @Autowired
    private JdRefundQualityCheckService qualityCheckService;

    @Autowired
    private StockService stockService;

    @Autowired
    private ProductInfoService productInfoService;

    @Override
    public Page<JdRefundNotQualityCheckVo> getPage(JdRefundNotQualityCheckSelectDto dto) {
        IWrapper<JdRefundNotQualityCheck> wrapper = getWrapper();
        wrapper.orderByDesc("jrnqc", JdRefundNotQualityCheck::getId);
        wrapper.gt("jrnqc", JdRefundNotQualityCheck::getQuantity, 0);
        Page<JdRefundNotQualityCheckVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @DSTransactional
    @Override
    public synchronized void submitQualityCheck(List<JdRefundQualityCheck> dto) {

        Map<Long, JdRefundQualityCheck> qualityCheckMap = dto.stream()
                .peek(item -> item.setId(null))
                .peek(item -> item.setQualifiedCount(ObjectUtil.defaultIfNull(item.getQualifiedCount(), 0)))
                .peek(item -> item.setDefectiveCount(ObjectUtil.defaultIfNull(item.getDefectiveCount(), 0)))
                .peek(item -> item.setScrappedCount(ObjectUtil.defaultIfNull(item.getScrappedCount(), 0)))
                .filter(item -> item.getQualifiedCount() + item.getDefectiveCount() + item.getScrappedCount() > 0)
                .collect(Collectors.toMap(JdRefundQualityCheck::getProductId, Function.identity()));

        if (qualityCheckMap.isEmpty()) {
            return;
        }

        // 获取质检产品
        Set<Long> productIdSet = qualityCheckMap.keySet();

        // 验证质检数
        List<JdRefundNotQualityCheck> list = list(q -> q.in(JdRefundNotQualityCheck::getProductId, productIdSet));
        Map<Long, JdRefundNotQualityCheck> notQualityCheckMap = list.stream().collect(Collectors.toMap(JdRefundNotQualityCheck::getProductId, Function.identity()));

        String code = CodeEnum.REFUND_QUALITY_CHECK.getCode();

        Collection<JdRefundQualityCheck> jdRefundQualityChecks = qualityCheckMap.values();
        for (JdRefundQualityCheck jdRefundQualityCheck : jdRefundQualityChecks) {

            jdRefundQualityCheck.setCode(code);

            // 产品id
            Long productId = jdRefundQualityCheck.getProductId();

            // 待质检数量
            JdRefundNotQualityCheck jdRefundNotQualityCheck = notQualityCheckMap.get(productId);

            int notCheckCount = jdRefundNotQualityCheck == null ? 0 : jdRefundNotQualityCheck.getQuantity();

            Integer qualifiedCount = jdRefundQualityCheck.getQualifiedCount();
            Integer defectiveCount = jdRefundQualityCheck.getDefectiveCount();
            Integer scrappedCount = jdRefundQualityCheck.getScrappedCount();

            int totalCheckCount = qualifiedCount + defectiveCount + scrappedCount;

            if (notCheckCount < totalCheckCount) {
                ProductInfo productInfo = productInfoService.getById(productId);

                throw new ServiceException("产品【" + productInfo.getCustomCode() + "】待质检数量为"
                        + notCheckCount + ", 当前质检数量为" + totalCheckCount);
            }

            // 修改待质检数量
            jdRefundNotQualityCheck.setQuantity(notCheckCount - totalCheckCount);

            // 修改库存
            stockService.update(q -> q
                    .eq(Stock::getProductId, jdRefundQualityCheck.getProductId())
                    .eq(Stock::getWarehouseId, WAREHOUSE_ID)
                    // 良品入库
                    .setSql(qualifiedCount > 0, "quantity = quantity + " + qualifiedCount)
                    // 次品入库
                    .setSql(defectiveCount > 0, getUpdateStockSql(3, defectiveCount))
                    // 减少锁定库存
                    .setSql(qualifiedCount + defectiveCount > 0, getUpdateStockSql(2, qualifiedCount + defectiveCount))
            );

        }

        // 保存质检记录
        qualityCheckService.saveBatch(jdRefundQualityChecks);

        // 修改待质检数量
        updateBatchById(list);
    }

}
